#include <afblib/inbuf.h>
#include <afblib/outbuf.h>
#include "mxprequest.h"

static int read_keyword(inbuf* ibuf, stralloc* keyword) {
   int ch;
   if (!stralloc_copys(keyword, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0 &&
         ch >= 'a' && ch <= 'z') {
      if (!stralloc_readyplus(keyword, 1)) return 0;
      keyword->s[keyword->len++] = ch;
   }
   if (ch >= 0) inbuf_back(ibuf);
   return keyword->len > 0;
}

static int read_parameter(inbuf* ibuf, stralloc* parameter) {
   int ch;
   if (!stralloc_copys(parameter, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0 &&
         ch != '\r' && ch != '\n' && ch != '/' && ch != '\0') {
      if (!stralloc_readyplus(parameter, 1)) return 0;
      parameter->s[parameter->len++] = ch;
   }
   if (ch >= 0) inbuf_back(ibuf);
   return parameter->len > 0;
}

static int expect_delimiter(inbuf* ibuf, char delimiter) {
   int ch = inbuf_getchar(ibuf);
   if (ch < 0) return 0;
   return ch == delimiter;
}

/* read one request from the given input buffer */
int read_mxp_request(inbuf* ibuf, mxp_request* request) {
   return
      read_keyword(ibuf, &request->keyword) &&
      expect_delimiter(ibuf, ' ') &&
      read_parameter(ibuf, &request->parameter) &&
      expect_delimiter(ibuf, '\r') &&
      expect_delimiter(ibuf, '\n');
}

/* write one request to the given outbuf buffer */
int write_mxp_request(outbuf* obuf, mxp_request* request) {
   return
      outbuf_write(obuf, request->keyword.s,
         request->keyword.len) == request->keyword.len &&
      outbuf_putchar(obuf, ' ') == ' ' &&
      outbuf_write(obuf, request->parameter.s,
         request->parameter.len) == request->parameter.len &&
      outbuf_putchar(obuf, '\r') == '\r' &&
      outbuf_putchar(obuf, '\n') == '\n';
}

/* release resources associated with request */
void free_mxp_request(mxp_request* request) {
   stralloc_free(&request->keyword);
   stralloc_free(&request->parameter);
}
