#include <afblib/inbuf.h>
#include <afblib/outbuf.h>
#include "mxpresponse.h"

static int read_message(inbuf* ibuf, stralloc* message) {
   int ch;
   if (!stralloc_copys(message, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0 &&
         ch != '\r' && ch != '\n') {
      if (!stralloc_readyplus(message, 1)) return 0;
      message->s[message->len++] = ch;
   }
   if (ch >= 0) inbuf_back(ibuf);
   return 1;
}

static int expect_delimiter(inbuf* ibuf, char delimiter) {
   int ch = inbuf_getchar(ibuf);
   if (ch < 0) return 0;
   return ch == delimiter;
}

/* read one (possibly partial) response from the given input buffer */
int read_mxp_response(inbuf* ibuf, mxp_response* response) {
   int ch = inbuf_getchar(ibuf);
   switch (ch) {
      case MXP_SUCCESS:
      case MXP_FAILURE:
      case MXP_CONTINUATION:
         response->status = ch;
         break;
      default:
         return 0;
   }
   if (!read_message(ibuf, &response->message)) return 0;
   if (!expect_delimiter(ibuf, '\r')) return 0;
   if (!expect_delimiter(ibuf, '\n')) return 0;
   return 1;
}

/* write one (possibly partial) response to the given output buffer */
int write_mxp_response(outbuf* obuf, mxp_response* response) {
   if (outbuf_putchar(obuf, response->status) < 0) return 0;
   if (response->message.len > 0) {
      ssize_t nbytes = outbuf_write(obuf, response->message.s,
         response->message.len);
      if (nbytes != response->message.len) return 0;
   }
   return
      outbuf_putchar(obuf, '\r') >= 0 &&
      outbuf_putchar(obuf, '\n') >= 0;
}

void free_mxp_response(mxp_response* response) {
   stralloc_free(&response->message);
}
